@@ -62,9 +62,9 @@ class Instrument:
 		dur = state["resolution"]
 		if state["swing"]:
 			if state["tick"] % 2 == 0:
-				dur *= (2.0 / 3.0)
+				dur = 1.0 / (1.0 / dur * (2.0 / 3.0))
 			else:
-				dur *= (4.0 / 3.0)
+				dur = 1.0 / (1.0 / dur * (4.0 / 3.0))
 		if len(self.bar) > 0:
 			if self.bar[0][2] is None:
 				i = 1 / (1.0 / dur + \
@@ -84,7 +84,6 @@ class Instrument:
 	
 		if not self.params['let_ring']:
 			return 1
-		
 		if 'note_length' in self.params:
 			return self.params["note_length"]
 
@@ -113,9 +112,9 @@ class Instrument:
 		dur = state["resolution"]
 		if state["swing"]:
 			if state["tick"] % 2 == 0:
-				dur *= (2.0 / 3.0)
+				dur = 1.0 / (1.0 / dur * (2.0 / 3.0))
 			else:
-				dur *= (4.0 / 3.0)
+				dur = 1.0 / (1.0 / dur * (4.0 / 3.0))
 
 		n = self.generate_note(state)
 		v = int(self.generate_velocity(state))
@@ -156,6 +155,7 @@ class Instrument:
 			self.bar.place_notes(notes, duration)	
 		else:
 			self.bar.bar[-1][2].add_notes(notes)
+			print "sdf?"
 
 	def record_rest(self, duration):
 		"""Adds a rest to the track, or adds a tick to the last\
@@ -163,7 +163,7 @@ NoteContainer if there are notes playing."""
 		dur = duration
 		if len(self.bar) > 0:
 			if self.bar[-1][1] is None or len(self.playing) > 0:
-				i = 1 / (1.0 / dur + 1.0 / self.bar[-1][1])
+				i = 1.0 / (1.0 / dur + 1.0 / self.bar[-1][1])
 				self.bar[-1][1] = i
 			else:
 				self.bar.place_notes(None, dur)
