@@ -8,6 +8,7 @@ from qtGUI.progressionDialog import Ui_progressionDialog
 from qtGUI.blockDialog import Ui_blockDialog
 from mingus.containers.Instrument import MidiInstrument
 from mingus.core import chords
+from mingus.core import progressions as mingus_progressions
 from sys import argv
 from os import path
 import Musicians
@@ -89,7 +90,7 @@ class InstrumentDialog(QtGui.QDialog):
 		instr_str = str(instr_str)
 		parts = instr_str.split(" ")
 
-		index = self.combo_index_by_text(self.ui.algorithm, parts[0])
+		index = combo_index_by_text(self.ui.algorithm, parts[0])
 		if index > 0:
 			self.ui.algorithm.setCurrentIndex(index)
 
@@ -132,11 +133,12 @@ class InstrumentDialog(QtGui.QDialog):
 			self.ui.midi.setCurrentIndex(0)
 
 
-	def combo_index_by_text(self, combo, text):
-		for x in range(combo.count()):
-			if combo.itemText(x) == text:
-				return x
-		return -1
+def combo_index_by_text(combo, text):
+	for x in range(combo.count()):
+		if combo.itemText(x) == text:
+			return x
+	return -1
+
 
 class ProgressionDialog(QtGui.QDialog):
 
@@ -172,23 +174,88 @@ class ProgressionDialog(QtGui.QDialog):
 			QtCore.SIGNAL("clicked()"),
 			self.add_progression)
 
+		self.connect(self.ui.updateprogression,
+			QtCore.SIGNAL("clicked()"),
+			lambda: self.add_progression(True))
+
 		self.connect(self.ui.buttonBox,
 			QtCore.SIGNAL("accepted()"),
 			lambda: self.save_progression())
 
-	def add_chord(self):
+		self.connect(self.ui.clearsequence, 
+			QtCore.SIGNAL("clicked()"),
+			self.ui.progressionsequence.clear)
+
+		self.connect(self.ui.removesequence,
+			QtCore.SIGNAL("clicked()"),
+			self.remove_sequence)
+
+		self.connect(self.ui.upsequence,
+			QtCore.SIGNAL("clicked()"),
+			lambda: self.swap_list_item(self.ui.progressionsequence))
+
+		self.connect(self.ui.downsequence,
+			QtCore.SIGNAL("clicked()"),
+			lambda: self.swap_list_item(self.ui.progressionsequence, False))
+
+		self.connect(self.ui.progressionsequence,
+			QtCore.SIGNAL("itemSelectionChanged()"),
+			self.show_progression)
+
+		self.connect(self.ui.updatechord,
+			QtCore.SIGNAL("clicked()"),
+			lambda: self.add_chord(True))
+
+		self.connect(self.ui.clearprogression, 
+			QtCore.SIGNAL("clicked()"),
+			self.ui.progression.clear)
+
+		self.connect(self.ui.removeprogression,
+			QtCore.SIGNAL("clicked()"),
+			self.remove_progression)
+
+		self.connect(self.ui.upprogression,
+			QtCore.SIGNAL("clicked()"),
+			lambda: self.swap_list_item(self.ui.progression))
+
+		self.connect(self.ui.downprogression,
+			QtCore.SIGNAL("clicked()"),
+			lambda: self.swap_list_item(self.ui.progression, False))
+
+		self.connect(self.ui.progression,
+			QtCore.SIGNAL("itemSelectionChanged()"),
+			self.show_chord)
+
+
+	def add_chord(self, update = False):
+		if update:
+			i = self.ui.progression.currentRow()
+			if i == -1:
+				return
+
 		chord = str(self.ui.accidentals.currentText())
 		chord += str(self.ui.romannumeral.currentText())
 		chord += str(self.ui.chordsuffix.currentText())
 		duration = str(self.ui.bars.currentText())
-		self.ui.progression.addItem("%s %s" % (duration, chord))
+		r=  "%s %s" % (duration, chord)
+		if not update:
+			self.ui.progression.addItem(r)
+		else:
+			self.ui.progression.item(i).setText(r)
+
 		self.ui.accidentals.setCurrentIndex(0)
 		self.ui.romannumeral.setCurrentIndex(0)
 		self.ui.chordsuffix.setCurrentIndex(0)
 		self.ui.bars.setCurrentIndex(3)
 
-	def add_progression(self):
+	def add_progression(self, update = False):
 		res=""
+
+		if update:
+			i = self.ui.progressionsequence.currentRow()
+			if i == -1:
+				return
+
 		for x in range(self.ui.progression.count()):
 			t = self.ui.progression.item(x).text()
 			parts = t.split(" ")
@@ -197,9 +264,13 @@ class ProgressionDialog(QtGui.QDialog):
 			else:
 				text = "%s%s" % (parts[0], parts[1])
 			res += "%s-" % text
+
 		if res != "":
-			r = str(self.ui.repeat.value())
-			self.ui.progressionsequence.addItem(r + " " + res[:-1])
+			r = str(self.ui.repeat.value() + 1) + " " + res[:-1]
+			if not update:
+				self.ui.progressionsequence.addItem(r)
+			else:
+				self.ui.progressionsequence.item(i).setText(r)
 
 	def load_progression(self, prog_str):
 		p = str(prog_str)
@@ -207,19 +278,23 @@ class ProgressionDialog(QtGui.QDialog):
 		if len(parts) == 0:
 			return 
 
+		if len(parts) == 1 and p in Options.get_available_progressions():
+			x = Options.progression_to_string(getattr(Progressions, p))
+			return self.load_progression("%s %s" % (p, x))
+
 		self.ui.progressionname.setText(parts[0])
 		if len(parts) > 3:
 			params = parts[2:-1]
 			for x in params:
 				r = x.split("*")
 				if len(r) == 2:
-					self.ui.progressionsequence.addItem("%d %s" % (int(r[0]) - 1, r[1]))
+					self.ui.progressionsequence.addItem("%d %s" % (int(r[0]), r[1]))
 				if len(r) == 1 and x != "":
 					self.ui.progressionsequence.addItem("0 %s" % r[0])
 
 	def save_progression(self):
 		res = ""
-		name = str(self.ui.progressionname.text())
+		name = str(self.ui.progressionname.text()).replace(" ", "_")
 		if name == "":
 			return
 		
@@ -227,11 +302,89 @@ class ProgressionDialog(QtGui.QDialog):
 		for x in range(self.ui.progressionsequence.count()):
 			t = self.ui.progressionsequence.item(x).text()
 			parts = t.split(" ")
-			res += "%d*%s " % (int(parts[0]) + 1, parts[1])
+			res += "%d*%s " % (int(parts[0]), parts[1])
 		res += " }"
 
 		self.item.setText(res)
 
+
+	def swap_list_item(self, lst, up = True):
+		index = lst.currentRow()
+		if index < 0:
+			return
+		c = lst.count()
+		cur = lst.item(index)
+		i1 = cur.text()
+		if up:
+			if index > 0:
+				i2 = lst.item(index - 1)
+				lst.setCurrentRow(index - 1)
+			else:
+				return 
+
+		else:
+			if index < c - 1:
+				i2 = lst.item(index + 1)
+				lst.setCurrentRow(index + 1)
+			else:
+				return
+		t = i2.text()
+		i2.setText(i1)
+		cur.setText(t)
+
+	def remove_sequence(self):
+		i = self.ui.progressionsequence.currentRow()
+		if i >= 0:
+			self.ui.progressionsequence.takeItem(i)
+
+	def remove_progression(self):
+		i = self.ui.progression.currentRow()
+		if i >= 0:
+			self.ui.progression.takeItem(i)
+
+	def show_progression(self):
+		i = self.ui.progressionsequence.currentRow()
+		if i >= 0:
+			self.ui.progression.clear()
+			prog = self.ui.progressionsequence.item(i).text()
+			p = prog.split(" ")
+			self.ui.repeat.setValue(int(p[0]) - 1)
+			chords = p[1].split("-")
+			for c in chords:
+				if type(c[0]) != int:
+					self.ui.progression.addItem("1 %s" % c)
+				else:
+					print "Can't parse bar length"
+
+	def show_chord(self):
+		i = self.ui.progression.currentRow()
+		if i >= 0:
+			parts = str(self.ui.progression.item(i).text()).split(" ")
+			self.ui.bars.setCurrentIndex(combo_index_by_text(
+				self.ui.bars, parts[0]))
+			chord =mingus_progressions.parse_string(parts[1])
+			self.ui.romannumeral.setCurrentIndex(combo_index_by_text(
+				self.ui.romannumeral, chord[0]))
+			self.ui.chordsuffix.setCurrentIndex(combo_index_by_text(
+				self.ui.chordsuffix, chord[2]))
+
+			acc = chord[1]
+			r = ''
+			while acc > 0:
+				r += '#'
+				acc -= 1
+			while acc < 0:
+				r += 'b'
+				acc += 1
+			self.ui.accidentals.setCurrentIndex(combo_index_by_text(
+				self.ui.accidentals, r))
+
+
+
+
+
+
+
 class BlockDialog(QtGui.QDialog):
 
 	def __init__(self, item):
@@ -508,10 +661,10 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 		self.ui.wildness.setValue(o.wild)
 
 		# ComboBoxes
-		i = self.combo_index_by_text(self.ui.resolution, str(o.resolution))
+		i = combo_index_by_text(self.ui.resolution, str(o.resolution))
 		if i != -1:
 			self.ui.resolution.setCurrentIndex(i)
-		i = self.combo_index_by_text(self.ui.key, o.key)
+		i = combo_index_by_text(self.ui.key, o.key)
 		if i != -1:
 			self.ui.key.setCurrentIndex(i)
 
@@ -574,12 +727,6 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 		o.midifile = self.get_file_name(o.project)
 		return o
 
-	def combo_index_by_text(self, combo, text):
-		for x in range(combo.count()):
-			if combo.itemText(x) == text:
-				return x
-		return -1
-
 	def swap_list_item(self, lst, up = True):
 		index = lst.currentRow()
 		if index < 0:
